#include <tizen.h>
#include <service_app.h>

#include <bundle.h>				// Header to use Bundle API
#include <message_port.h>		// Header to use Message Port API

#define TAG "bundle"			// Log TAG
#include "bundlesendernativeservice.h"

void
sendMessage(char *remote_app_id,char *remote_port,bundle *reply){

	int ret = message_port_send_message(remote_app_id, remote_port, reply);	 //send 'reply' bundle to the remote port

	if (ret != MESSAGE_PORT_ERROR_NONE)
	    dlog_print(DLOG_ERROR, TAG, "message_port_check_remote_port error: %d", ret);
	else
	    dlog_print(DLOG_INFO, TAG, "Send message done");
}

void
test_check_remote_port(char *remote_app_id,char *remote_port,bool *result)
{
    int ret;

    ret = message_port_check_remote_port(remote_app_id,remote_port,result);		// Function to check if the remote port exists
    dlog_print(DLOG_DEBUG, TAG ,"Remote app id: %s Remote port: %s Result: %d", remote_app_id, remote_port, *result);

    if (ret != MESSAGE_PORT_ERROR_NONE)
        dlog_print(DLOG_ERROR, TAG, "message_port_check_remote_port error: %d", ret);
}

void requestRemotePort()
{
	char *remote_app_id="G85Cygkpsb.BundleReceiverWebUI";	// Web app id (look in config.xml)
	char *remote_port="CrossPort";							// Remote message port id (Name of Local port in Web app)
	bool result=false;

	test_check_remote_port(remote_app_id,remote_port,&result);
	dlog_print(DLOG_DEBUG, TAG ,"Remote port exists: %s", result ? "true" : "false");

	bundle *reply = NULL;
	reply=bundle_create();					// Create bundle instance

	bundle_add_str(reply,"testChar","T");	// Add string key Value pair to bundle


	char *array[] = {"Z2","Z4","GearS2","GearS3"};
	bundle_add_str_array(reply,"Models",array, 4);		// Add string array of key Value pair to bundle, length of 4

	dlog_print(DLOG_DEBUG, TAG, "Bundled Successfully");


	if (result){
		sendMessage(remote_app_id,remote_port,reply);
		bundle_free(reply);				// Destroy the bundle instance when not required anymore
	}

}


bool service_app_create(void *data)
{
	requestRemotePort();	// Invoke function for Message Port Everytime the Service app is created, Move the call to your desired position
	dlog_print(DLOG_DEBUG, TAG, "Service app Launched");
    return true;
}

void service_app_terminate(void *data)
{
    // Todo: add your code here.
	dlog_print(DLOG_DEBUG, TAG, "Service app Terminated");
    return;
}

void service_app_control(app_control_h app_control, void *data)
{
    // Todo: add your code here.
    return;
}

static void
service_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	return;
}

static void
service_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

static void
service_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}

static void
service_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

int main(int argc, char* argv[])
{
    char ad[50] = {0,};
	service_app_lifecycle_callback_s event_callback;
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = service_app_create;
	event_callback.terminate = service_app_terminate;
	event_callback.app_control = service_app_control;

	service_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, service_app_low_battery, &ad);
	service_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, service_app_low_memory, &ad);
	service_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, service_app_lang_changed, &ad);
	service_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, service_app_region_changed, &ad);

	return service_app_main(argc, argv, &event_callback, ad);
}
